#!/usr/bin/perl -w
#
# Parse the outputs from Nagios' avail.cgi into an HTML table.
#
# Chet Luther
# <chet@rcn.com>

use LWP::UserAgent;
use Time::Local;
use Data::Dumper;

use vars qw($agent);
use strict;

my $nagios_url = "http://ncc.cmg.fcnbd.com/nagios/cgi-bin";

$agent = new LWP::UserAgent();
$agent->agent("Mozilla/6.0");
$agent->timeout(600);

my @hostgroups;
add_hostgroup("CMG-Firewall Cluster",	"CMG-Firewall Cluster");
add_hostgroup("CMG-Vendor Cluster",	"CMG-Vendor Cluster");
add_hostgroup("Core-Routers",		"Core-Routers");
add_hostgroup("Core-Switches",		"Core-Switches");
add_hostgroup("Regional-Routers",	"Regional-Routers");
add_hostgroup("Remote-Access",		"Remote-Access");
add_hostgroup("Vendors",		"Vendors");
add_hostgroup("switches",		"switches");
add_hostgroup("users",		"regional users");
add_hostgroup("elgin-dr",		"elgin-dr");
add_hostgroup("fw1-stations",		"fw1-stations");
add_hostgroup("Lotus-Notes",		"Lotus-Notes");
add_hostgroup("Internet Web-Sites",		"Internet Web-Sites");

sub main; {
  my $date = $ARGV[0] || get_yesterday();

  my $output;
  $output .= "<table border=\"1\" cellspacing=\"1\" cellpadding=\"3\">\n";
  $output .= "<tr>\n";
  $output .= " <th>$date</th>\n";
  $output .= " <th align=\"center\"><b>Last 24 Hours</b></th>\n";
  $output .= " <th align=\"center\"><b>Last 7 Days</b></th>\n";
  $output .= " <th align=\"center\"><b>Last 30 Days</b></th>\n";
  $output .= " <th align=\"center\"><b>Last 365 Days</b></th>\n";
  $output .= "</tr>\n";

  my @timeperiods;
  for (my $i = 0; $i <= $#hostgroups; $i++) {
    if (my $availability = get_avail($hostgroups[$i], $date)) {
      $output .= "<tr>\n";
      $output .= " <td align=\"right\">$hostgroups[$i]->{desc}</td>\n";
      $output .= sprintf(" <td align=\"center\"><a href=\"$nagios_url/avail.cgi?t1=$availability->{day_epoch}\&t2=$availability->{end_epoch}\&hostgroup=$hostgroups[$i]->{alias}\&backtrack=3\">%.2f%%</a></td>\n", $availability->{day});
      $output .= sprintf(" <td align=\"center\"><a href=\"$nagios_url/avail.cgi?t1=$availability->{week_epoch}\&t2=$availability->{end_epoch}\&hostgroup=$hostgroups[$i]->{alias}\&backtrack=4\">%.2f%%</a></td>\n", $availability->{week});
      $output .= sprintf(" <td align=\"center\"><a href=\"$nagios_url/avail.cgi?t1=$availability->{month_epoch}\&t2=$availability->{end_epoch}\&hostgroup=$hostgroups[$i]->{alias}\&backtrack=5\">%.2f%%</a></td>\n", $availability->{month});
      $output .= sprintf(" <td align=\"center\"><a href=\"$nagios_url/avail.cgi?t1=$availability->{year_epoch}\&t2=$availability->{end_epoch}\&hostgroup=$hostgroups[$i]->{alias}\&backtrack=60\">%.2f%%</a></td>\n", $availability->{year});
      $output .= "</tr>\n";

      $timeperiods[0] += $availability->{day};
      $timeperiods[1] += $availability->{week};
      $timeperiods[2] += $availability->{month};
      $timeperiods[3] += $availability->{year};
    } else {
      splice(@hostgroups, $i--, 1);
    }
  }

  foreach my $timeperiod (@timeperiods) { $timeperiod = $timeperiod / ($#hostgroups+1); }

  $output .= "<tr>\n";
  $output .= " <th align=\"right\"><b>Average</b></th>\n";
  $output .= sprintf(" <th align=\"center\"><b>%.2f%%</b></td>\n", $timeperiods[0]);
  $output .= sprintf(" <th align=\"center\"><b>%.2f%%</b></td>\n", $timeperiods[1]);
  $output .= sprintf(" <th align=\"center\"><b>%.2f%%</b></td>\n", $timeperiods[2]);
  $output .= sprintf(" <th align=\"center\"><b>%.2f%%</b></td>\n", $timeperiods[3]);
  $output .= "</tr>\n";

  $output .= "</table>\n";
  print $output;
}

sub get_avail { my ($hostgroup, $std_date) = @_;
  my @start_epoch = (
    get_epoch($std_date)-86400,
    get_epoch($std_date)-604800,
    get_epoch($std_date)-2592000,
    get_epoch($std_date)-31536000
  );

  my $end_epoch = get_epoch($std_date);

  my @timeperiods;
  for (my $i = 0; $i <= $#start_epoch; $i++) {
    my $backtrack;
    if    ($i == 0) { $backtrack = 1; }
    elsif ($i == 1) { $backtrack = 2; }
    elsif ($i == 2) { $backtrack = 5; }
    elsif ($i == 3) { $backtrack = 53; }

    my $request = new HTTP::Request(GET => "$nagios_url/avail.cgi?t1=$start_epoch[$i]\&t2=$end_epoch\&hostgroup=$hostgroup->{alias}\&backtrack=$backtrack");

    $request->header('Accept' => 'text/html');
    $request->authorization_basic('guest', 'guest');

    my $response = $agent->request($request);

    if (!$response->is_success()) {
      exit;
    }

    my @down_percents;
    foreach my $line (split(/\n/, $response->content())) {
      if ($line =~ /^<tr CLASS=\'data/) {
        $_ = $line; /hostDOWN\'>\d+\.\d+%/; $_ = $&; /\d+\.\d+/;
        push(@down_percents, $&);
      }
    }

    # return(0) if ($#down_percents == -1);
    if ($#down_percents == -1) {
      return 0;
    }

    my $total = "0.00";
    foreach my $percent (@down_percents) { $total += $percent; }

    push(@timeperiods, 100-($total / ($#down_percents + 1)));
  }

  my %avail = (
    day   => $timeperiods[0],
    week  => $timeperiods[1],
    month => $timeperiods[2],
    year  => $timeperiods[3],
    end_epoch => $end_epoch,
    day_epoch => $start_epoch[0],
    week_epoch => $start_epoch[1],
    month_epoch => $start_epoch[2],
    year_epoch => $start_epoch[3]
  );

  return(\%avail);
}

sub add_hostgroup { my ($alias, $desc) = @_;
  my %hostgroup;

  $hostgroup{alias} = $alias;
  $hostgroup{desc} = $desc;

  push(@hostgroups, \%hostgroup);
}

sub get_yesterday {
  my @list_yesterday = localtime((time() - 86400));
  $list_yesterday[4] += 1;
  $list_yesterday[5] += 1900;

  my $yesterday = sprintf("%04d.%02d.%02d",
                          $list_yesterday[5],
                          $list_yesterday[4],
                          $list_yesterday[3]);

  return($yesterday);
}

sub get_epoch { my $std_time = $_[0];
  my @delim = split(/\./, $std_time);
  my $epoch = timelocal(59, 59, 23, $delim[2], $delim[1]-1, $delim[0]-1900);

  return($epoch);
}
